package kerho;

import java.util.List;

/**
 * Kerho-luokka, joka huolehtii jäsenistöstä.  Pääosin kaikki metodit
 * ovat vain "välittäjämetodeja" jäsenistöön.
 *
 * @author Vesa Lappalainen
 * @version 1.0, 09.02.2003
 * @version 1.1, 23.02.2003
 * @version 1.2, 07.01.2008 / testit
 * @version 1.3, 03.03.2013 / Harrastukset
 */
public class Kerho {
    private final Jasenet jasenet = new Jasenet();
    private final Harrastukset harrastukset = new Harrastukset(); 


    /**
     * Palautaa kerhon jäsenmäärän
     * @return jäsenmäärä
     */
    public int getJasenia() {
        return jasenet.getLkm();
    }


    /**
     * Poistaa jäsenistöstä ja harrasteista ne joilla on nro. Kesken.
     * @param nro viitenumero, jonka mukaan poistetaan
     * @return montako jäsentä poistettiin
     */
    public int poista(@SuppressWarnings("unused") int nro) {
        return 0;
    }


    /**
     * Lisää kerhoon uuden jäsenen
     * @param jasen lisättävä jäsen
     * @throws SailoException jos lisäystä ei voida tehdä
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Kerho kerho = new Kerho();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * aku1.rekisteroi(); aku2.rekisteroi();
     * kerho.getJasenia() === 0;
     * kerho.lisaa(aku1); kerho.getJasenia() === 1;
     * kerho.lisaa(aku2); kerho.getJasenia() === 2;
     * kerho.lisaa(aku1); kerho.getJasenia() === 3;
     * kerho.getJasenia() === 3;
     * kerho.annaJasen(0) === aku1;
     * kerho.annaJasen(1) === aku2;
     * kerho.annaJasen(2) === aku1;
     * kerho.annaJasen(3) === aku1; #THROWS IndexOutOfBoundsException 
     * kerho.lisaa(aku1); kerho.getJasenia() === 4;
     * kerho.lisaa(aku1); kerho.getJasenia() === 5;
     * kerho.lisaa(aku1);            #THROWS SailoException
     * </pre>
     */
    public void lisaa(Jasen jasen) throws SailoException {
        jasenet.lisaa(jasen);
    }


    /**
     * Listään uusi harrastus kerhoon
     * @param har lisättävä harrastus 
     */
    public void lisaa(Harrastus har) {
        harrastukset.lisaa(har);
    }


    /**
     * Palauttaa i:n jäsenen
     * @param i monesko jäsen palautetaan
     * @return viite i:teen jäseneen
     * @throws IndexOutOfBoundsException jos i väärin
     */
    public Jasen annaJasen(int i) throws IndexOutOfBoundsException {
        return jasenet.anna(i);
    }


    /**
     * Haetaan kaikki jäsen harrastukset
     * @param jasen jäsen jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Kerho kerho = new Kerho();
     *  Jasen aku1 = new Jasen(), aku2 = new Jasen(), aku3 = new Jasen();
     *  aku1.rekisteroi(); aku2.rekisteroi(); aku3.rekisteroi();
     *  int id1 = aku1.getTunnusNro();
     *  int id2 = aku2.getTunnusNro();
     *  Harrastus pitsi11 = new Harrastus(id1); kerho.lisaa(pitsi11);
     *  Harrastus pitsi12 = new Harrastus(id1); kerho.lisaa(pitsi12);
     *  Harrastus pitsi21 = new Harrastus(id2); kerho.lisaa(pitsi21);
     *  Harrastus pitsi22 = new Harrastus(id2); kerho.lisaa(pitsi22);
     *  Harrastus pitsi23 = new Harrastus(id2); kerho.lisaa(pitsi23);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = kerho.annaHarrastukset(aku3);
     *  loytyneet.size() === 0; 
     *  loytyneet = kerho.annaHarrastukset(aku1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == pitsi21 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }
    

    /**
     * Lukee kerhon tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        jasenet.lueTiedostosta(nimi);
        harrastukset.lueTiedostosta(nimi);
    }


    /**
     * Tallettaa kerhon tiedot tiedostoon
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void talleta() throws SailoException {
        jasenet.talleta();
        harrastukset.talleta();
        // TODO: yritä tallettaa toinen vaikka toinen epäonnistuisi
    }


    /**
     * Testiohjelma kerhosta
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Kerho kerho = new Kerho();

        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen aku1 = new Jasen(), aku2 = new Jasen();
            aku1.rekisteroi();
            aku1.vastaaAkuAnkka();
            aku2.rekisteroi();
            aku2.vastaaAkuAnkka();

            kerho.lisaa(aku1);
            kerho.lisaa(aku2);
            int id1 = aku1.getTunnusNro();
            int id2 = aku2.getTunnusNro();
            Harrastus pitsi11 = new Harrastus(id1); pitsi11.vastaaPitsinNyplays(id1); kerho.lisaa(pitsi11);
            Harrastus pitsi12 = new Harrastus(id1); pitsi12.vastaaPitsinNyplays(id1); kerho.lisaa(pitsi12);
            Harrastus pitsi21 = new Harrastus(id2); pitsi21.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi21);
            Harrastus pitsi22 = new Harrastus(id2); pitsi22.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi22);
            Harrastus pitsi23 = new Harrastus(id2); pitsi23.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi23);

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < kerho.getJasenia(); i++) {
                Jasen jasen = kerho.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                List<Harrastus> loytyneet = kerho.annaHarrastukset(jasen);
                for (Harrastus harrastus : loytyneet)
                    harrastus.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
    }

}
